#version 450
#extension GL_EXT_nonuniform_qualifier: require
#extension GL_EXT_debug_printf: enable

layout (set = 0, binding = 0) uniform sampler2D textures[];

struct MaterialGpuData
{
    vec4 mDiffuseReflectivity;
    vec4 mAmbientReflectivity;
    vec4 mSpecularReflectivity;
    vec4 mEmissiveColor;
    vec4 mTransparentColor;
    vec4 mReflectiveColor;
    vec4 mAlbedo;

    float mOpacity;
    float mBumpScaling;
    float mShininess;
    float mShininessStrength;

    float mRefractionIndex;
    float mReflectivity;
    float mMetallic;
    float mSmoothness;

    float mSheen;
    float mThickness;
    float mRoughness;
    float mAnisotropy;

    vec4 mAnisotropyRotation;
    vec4 mCustomData;

    int mDiffuseTexIndex;
    int mSpecularTexIndex;
    int mAmbientTexIndex;
    int mEmissiveTexIndex;
    int mHeightTexIndex;
    int mNormalsTexIndex;
    int mShininessTexIndex;
    int mOpacityTexIndex;
    int mDisplacementTexIndex;
    int mReflectionTexIndex;
    int mLightmapTexIndex;
    int mExtraTexIndex;

    vec4 mDiffuseTexOffsetTiling;
    vec4 mSpecularTexOffsetTiling;
    vec4 mAmbientTexOffsetTiling;
    vec4 mEmissiveTexOffsetTiling;
    vec4 mHeightTexOffsetTiling;
    vec4 mNormalsTexOffsetTiling;
    vec4 mShininessTexOffsetTiling;
    vec4 mOpacityTexOffsetTiling;
    vec4 mDisplacementTexOffsetTiling;
    vec4 mReflectionTexOffsetTiling;
    vec4 mLightmapTexOffsetTiling;
    vec4 mExtraTexOffsetTiling;
};

layout (set = 1, binding = 0) buffer Material
{
    MaterialGpuData materials[];
} matSsbo;

layout (push_constant) uniform PushConstants {
    int mMaterialIndex;
} pushConstants;

layout (location = 0) in vec3 fragPosition;// World position from vert shader
layout (location = 1) in vec2 fragTexCoord;// Texture coordinate
layout (location = 2) in vec3 fragNormal;// Normal vector
layout (location = 3) in vec3 fragTangent;
layout (location = 4) in vec3 fragBitangent;

layout (location = 0) out vec4 gPosition;// G-Buffer Position
layout (location = 1) out vec4 gAlbedo;// G-Buffer Albedo
layout (location = 2) out vec4 gNormal;// G-Buffer Normal


void main() {
    MaterialGpuData material = matSsbo.materials[pushConstants.mMaterialIndex];

    mat3 TBN = mat3(normalize(fragTangent), normalize(fragBitangent), normalize(fragNormal));
    vec3 normalTS = texture(textures[material.mNormalsTexIndex], fragTexCoord).rgb;
    normalTS = normalize(normalTS * 2.0 - 1.0);
    vec3 normal = normalize(TBN * normalTS);

    gPosition = vec4(fragPosition, 1.0);
    gNormal = vec4(normal, 1.0);
    gAlbedo = texture(textures[material.mDiffuseTexIndex], fragTexCoord);
}
