#version 450

layout (location = 0) in vec3 inPosition;   // Vertex position
layout (location = 1) in vec2 inTexCoord;  // Texture coordinate
layout (location = 2) in vec3 inNormal;    // Normal vector
layout (location = 3) in vec3 inTangent;
layout (location = 4) in vec3 inBitangent;

layout (location = 0) out vec3 fragPosition; // World position to pass to frag
layout (location = 1) out vec2 fragTexCoord; // Texture coordinate
layout (location = 2) out vec3 fragNormal;   // Normal vector
layout (location = 3) out vec3 fragTangent;
layout (location = 4) out vec3 fragBitangent;

layout (set = 0, binding = 1) uniform CameraTransform {
    mat4 mViewProjMatrix; // Combined view-projection matrix
} ubo;

layout (set = 0, binding = 2) uniform ModelMatrices {
    mat4 mModelMatrix;
    mat4 mNormalMatrix;
} modelMatrices;

void main() {
    vec4 worldPosition = modelMatrices.mModelMatrix * vec4(inPosition.xyz, 1.0);
    fragPosition = worldPosition.xyz; // Pass world position

    fragTexCoord = inTexCoord;        // Pass texture coordinates

    fragNormal = mat3(modelMatrices.mNormalMatrix) * inNormal;
    fragTangent = mat3(modelMatrices.mNormalMatrix) * inTangent;
    fragBitangent = mat3(modelMatrices.mNormalMatrix) * inBitangent;

    vec4 clipPosition = ubo.mViewProjMatrix * worldPosition; // Transform to clip space
    gl_Position = clipPosition; // Output final position for rasterization

}
